@@ -1,7 +1,8 @@
 --- Windows implementation of filesystem and platform abstractions.
 -- Download http://unxutils.sourceforge.net/ for Windows GNU utilities
 -- used by this module.
-module("luarocks.fs.win32", package.seeall)
+--module("luarocks.fs.win32", package.seeall)
+local win32 = {}
 
 local fs = require("luarocks.fs")
 
@@ -17,14 +18,14 @@ local _popen, _execute = io.popen, os.execute
 io.popen = function(cmd, ...) return _popen(_prefix..cmd, ...) end
 os.execute = function(cmd, ...) return _execute(_prefix..cmd, ...) end
 
-
 --- Annotate command string for quiet execution.
 -- @param cmd string: A command-line string.
 -- @return string: The command-line, with silencing annotation.
-function quiet(cmd)
+function win32.quiet(cmd)
    return cmd.." 2> NUL 1> NUL"
 end
 
+local drive_letter = "[%.a-zA-Z]?:?[\\/]"
 
 local win_escape_chars = {
   ["%"] = "%%",
@@ -43,10 +44,10 @@ end
 -- Adds double quotes and escapes.
 -- @param arg string: Unquoted argument.
 -- @return string: Quoted argument.
-function Q(arg)
+function win32.Q(arg)
    assert(type(arg) == "string")
    -- Quote DIR for Windows
-   if arg:match("^[%.a-zA-Z]?:?[\\/]")  then
+   if arg:match("^"..drive_letter)  then
       arg = arg:gsub("/", "\\")
    end
    if arg == "\\" then
@@ -64,10 +65,10 @@ end
 -- Adds double quotes and escapes.
 -- @param arg string: Unquoted argument.
 -- @return string: Quoted argument.
-function Qb(arg)
+function win32.Qb(arg)
    assert(type(arg) == "string")
    -- Quote DIR for Windows
-   if arg:match("^[%.a-zA-Z]?:?[\\/]")  then
+   if arg:match("^"..drive_letter)  then
       arg = arg:gsub("/", "\\")
    end
    if arg == "\\" then
@@ -86,20 +87,26 @@ end
 -- pathname absolute, or the current dir in the dir stack if
 -- not given.
 -- @return string: The pathname converted to absolute.
-function absolute_name(pathname, relative_to)
+function win32.absolute_name(pathname, relative_to)
    assert(type(pathname) == "string")
    assert(type(relative_to) == "string" or not relative_to)
 
    relative_to = relative_to or fs.current_dir()
-   -- FIXME I'm not sure this first \\ should be there at all.
-   -- What are the Windows rules for drive letters?
-   if pathname:match("^[\\.a-zA-Z]?:?[\\/]") then
+   if pathname:match("^"..drive_letter) then
       return pathname
    else
       return relative_to .. "/" .. pathname
    end
 end
 
+--- Return the root directory for the given path.
+-- For example, for "c:\hello", returns "c:\"
+-- @param pathname string: pathname to use.
+-- @return string: The root of the given pathname.
+function win32.root_of(pathname)
+   return (fs.absolute_name(pathname):match("^("..drive_letter..")"))
+end
+
 --- Create a wrapper to make a script executable from the command-line.
 -- @param file string: Pathname of script to be made executable.
 -- @param dest string: Directory where to put the wrapper.
@@ -107,7 +114,7 @@ end
 -- @param version string: rock version to be used in loader context.
 -- @return boolean or (nil, string): True if succeeded, or nil and
 -- an error message.
-function wrap_script(file, dest, name, version)
+function win32.wrap_script(file, dest, name, version)
    assert(type(file) == "string")
    assert(type(dest) == "string")
 
@@ -124,11 +131,12 @@ function wrap_script(file, dest, name, version)
    local ppaths = "package.path="..util.LQ(lpath..";").."..package.path; package.cpath="..util.LQ(lcpath..";").."..package.cpath"
    local addctx = "local k,l,_=pcall(require,"..util.LQ("luarocks.loader")..") _=k and l.add_context("..util.LQ(name)..","..util.LQ(version)..")"
    wrapper:write(fs.Qb(lua)..' -e '..fs.Qb(ppaths)..' -e '..fs.Qb(addctx)..' '..fs.Qb(file)..' %*\n')
+   wrapper:write("exit /b %ERRORLEVEL%\n")
    wrapper:close()
    return true
 end
 
-function is_actual_binary(name)
+function win32.is_actual_binary(name)
    name = name:lower()
    if name:match("%.bat$") or name:match("%.exe$") then
       return true
@@ -136,14 +144,14 @@ function is_actual_binary(name)
    return false
 end
 
-function copy_binary(filename, dest) 
+function win32.copy_binary(filename, dest) 
    local ok, err = fs.copy(filename, dest)
    if not ok then
       return nil, err
    end
    local exe_pattern = "%.[Ee][Xx][Ee]$"
    local base = dir.base_name(filename)
-   local dest = dir.dir_name(dest)
+   dest = dir.dir_name(dest)
    if base:match(exe_pattern) then
       base = base:gsub(exe_pattern, ".lua")
       local helpname = dest.."/"..base
@@ -158,11 +166,11 @@ function copy_binary(filename, dest)
    return true
 end
 
-function chmod(filename, mode)
+function win32.chmod(filename, mode)
    return true
 end
 
-function get_permissions(filename)
+function win32.get_permissions(filename)
    return ""
 end
 
@@ -177,7 +185,7 @@ end
 -- which will replace old_file.
 -- @return boolean or (nil, string): True if succeeded, or nil and
 -- an error message.
-function replace_file(old_file, new_file)
+function win32.replace_file(old_file, new_file)
    os.remove(old_file)
    return os.rename(new_file, old_file)
 end
@@ -188,7 +196,7 @@ end
 -- for checking the result of subsequent operations.
 -- @param file string: filename to test
 -- @return boolean: true if file exists, false otherwise.
-function is_writable(file)
+function win32.is_writable(file)
    assert(file)
    file = dir.normalize(file)
    local result
@@ -212,3 +220,9 @@ function is_writable(file)
    end
    return result
 end
+
+function win32.tmpname()
+   return os.getenv("TMP")..os.tmpname()
+end
+
+return win32
