@@ -7,7 +7,7 @@ site_config.LUAROCKS_SYSCONFDIR=[[/etc/luarocks]]
 site_config.LUAROCKS_ROCKS_TREE=[[/usr]]
 site_config.LUAROCKS_ROCKS_SUBDIR=[[/lib/luarocks/rocks-5.3]]
 site_config.LUAROCKS_UNAME_S=[[Linux]]
-site_config.LUAROCKS_UNAME_M=[[armv5tel]]
+site_config.LUAROCKS_UNAME_M=[[armv7l]]
 site_config.LUAROCKS_DOWNLOADER=[[curl]]
 site_config.LUAROCKS_MD5CHECKER=[[md5sum]]
 return site_config
